/**
 * 
 *   Copyright (c) 2010-2013, Abel Hegedus, Zoltan Ujhelyi, Denes Harmath, Istvan Rath and Daniel Varro, IncQuery Labs Ltd.
 *   This program and the accompanying materials are made available under the
 *   terms of the Eclipse Public License v. 2.0 which is available at
 *   http://www.eclipse.org/legal/epl-v20.html.
 *   
 *   SPDX-License-Identifier: EPL-2.0
 *  
 */
package org.eclipse.viatra.integration.uml.derivedfeatures;

import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.viatra.integration.uml.derivedfeatures.internal.MessageReceiveEvent;
import org.eclipse.viatra.integration.uml.derivedfeatures.internal.MessageSendEvent;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFQuerySpecificationWithGenericMatcher;
import org.eclipse.viatra.query.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.emf.types.EDataTypeInSlotsKey;
import org.eclipse.viatra.query.runtime.matchers.backend.QueryEvaluationHint;
import org.eclipse.viatra.query.runtime.matchers.psystem.PBody;
import org.eclipse.viatra.query.runtime.matchers.psystem.PVariable;
import org.eclipse.viatra.query.runtime.matchers.psystem.annotations.PAnnotation;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.NegativePatternCall;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.ConstantValue;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.PositivePatternCall;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameterDirection;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PVisibility;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuples;

/**
 * A pattern-specific query specification that can instantiate GenericPatternMatcher in a type-safe way.
 * 
 * <p>Original source:
 *         <code><pre>
 *         MessageOperations.getMessageKind
 *         
 *         {@literal @}Surrogate(feature = "messageKind")
 *         pattern messageMessageKind(self: Message, messageKind: MessageKind) {
 *             neg find messageSendEvent(self, _);
 *             neg find messageReceiveEvent(self, _);
 *             messageKind == MessageKind::unknown;
 *         } or {
 *             neg find messageSendEvent(self, _);
 *             find messageReceiveEvent(self, _);
 *             messageKind == MessageKind::found;
 *         } or {
 *             find messageSendEvent(self, _);
 *             neg find messageReceiveEvent(self, _);
 *             messageKind == MessageKind::lost;
 *         } or {
 *             find messageSendEvent(self, _);
 *             find messageReceiveEvent(self, _);
 *             messageKind == MessageKind::complete;
 *         }
 * </pre></code>
 * 
 * @see GenericPatternMatcher
 * @see GenericPatternMatch
 * 
 */
@SuppressWarnings("all")
public final class MessageMessageKind extends BaseGeneratedEMFQuerySpecificationWithGenericMatcher {
  private MessageMessageKind() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws ViatraQueryRuntimeException if the pattern definition could not be loaded
   * 
   */
  public static MessageMessageKind instance() {
    try{
        return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
        throw processInitializerError(err);
    }
  }
  
  /**
   * Inner class allowing the singleton instance of {@link JvmGenericType: org.eclipse.viatra.integration.uml.derivedfeatures.MessageMessageKind (visibility: PUBLIC, simpleName: MessageMessageKind, identifier: org.eclipse.viatra.integration.uml.derivedfeatures.MessageMessageKind, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.integration.uml.derivedfeatures) (interface: false, strictFloatingPoint: false, anonymous: false)} to be created 
   *     <b>not</b> at the class load time of the outer class, 
   *     but rather at the first call to {@link JvmGenericType: org.eclipse.viatra.integration.uml.derivedfeatures.MessageMessageKind (visibility: PUBLIC, simpleName: MessageMessageKind, identifier: org.eclipse.viatra.integration.uml.derivedfeatures.MessageMessageKind, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.integration.uml.derivedfeatures) (interface: false, strictFloatingPoint: false, anonymous: false)#instance()}.
   * 
   * <p> This workaround is required e.g. to support recursion.
   * 
   */
  private static class LazyHolder {
    private static final MessageMessageKind INSTANCE = new MessageMessageKind();
    
    /**
     * Statically initializes the query specification <b>after</b> the field {@link #INSTANCE} is assigned.
     * This initialization order is required to support indirect recursion.
     * 
     * <p> The static initializer is defined using a helper field to work around limitations of the code generator.
     * 
     */
    private static final Object STATIC_INITIALIZER = ensureInitialized();
    
    public static Object ensureInitialized() {
      INSTANCE.ensureInitializedInternal();
      return null;
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private static final MessageMessageKind.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    private final PParameter parameter_self = new PParameter("self", "org.eclipse.uml2.uml.Message", new EClassTransitiveInstancesKey((EClass)getClassifierLiteralSafe("http://www.eclipse.org/uml2/5.0.0/UML", "Message")), PParameterDirection.INOUT);
    
    private final PParameter parameter_messageKind = new PParameter("messageKind", "org.eclipse.uml2.uml.MessageKind", new EDataTypeInSlotsKey((EDataType)getClassifierLiteralSafe("http://www.eclipse.org/uml2/5.0.0/UML", "MessageKind")), PParameterDirection.INOUT);
    
    private final List<PParameter> parameters = Arrays.asList(parameter_self, parameter_messageKind);
    
    private GeneratedPQuery() {
      super(PVisibility.PUBLIC);
    }
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.integration.uml.derivedfeatures.messageMessageKind";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("self","messageKind");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return parameters;
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() {
      setEvaluationHints(new QueryEvaluationHint(null, QueryEvaluationHint.BackendRequirement.UNSPECIFIED));
      Set<PBody> bodies = new LinkedHashSet<>();
      {
          PBody body = new PBody(this);
          PVariable var_self = body.getOrCreateVariableByName("self");
          PVariable var_messageKind = body.getOrCreateVariableByName("messageKind");
          PVariable var___0_ = body.getOrCreateVariableByName("_<0>");
          PVariable var___1_ = body.getOrCreateVariableByName("_<1>");
          new TypeConstraint(body, Tuples.flatTupleOf(var_self), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Message")));
          new TypeConstraint(body, Tuples.flatTupleOf(var_messageKind), new EDataTypeInSlotsKey((EDataType)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "MessageKind")));
          body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
             new ExportedParameter(body, var_self, parameter_self),
             new ExportedParameter(body, var_messageKind, parameter_messageKind)
          ));
          //     neg find messageSendEvent(self, _)
          new NegativePatternCall(body, Tuples.flatTupleOf(var_self, var___0_), MessageSendEvent.instance().getInternalQueryRepresentation());
          //     neg find messageReceiveEvent(self, _)
          new NegativePatternCall(body, Tuples.flatTupleOf(var_self, var___1_), MessageReceiveEvent.instance().getInternalQueryRepresentation());
          //     messageKind == MessageKind::unknown
          PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
          new ConstantValue(body, var__virtual_0_, org.eclipse.uml2.uml.MessageKind.get("unknown"));
          new Equality(body, var_messageKind, var__virtual_0_);
          bodies.add(body);
      }
      {
          PBody body = new PBody(this);
          PVariable var_self = body.getOrCreateVariableByName("self");
          PVariable var_messageKind = body.getOrCreateVariableByName("messageKind");
          PVariable var___0_ = body.getOrCreateVariableByName("_<0>");
          PVariable var___1_ = body.getOrCreateVariableByName("_<1>");
          new TypeConstraint(body, Tuples.flatTupleOf(var_self), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Message")));
          new TypeConstraint(body, Tuples.flatTupleOf(var_messageKind), new EDataTypeInSlotsKey((EDataType)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "MessageKind")));
          body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
             new ExportedParameter(body, var_self, parameter_self),
             new ExportedParameter(body, var_messageKind, parameter_messageKind)
          ));
          //     neg find messageSendEvent(self, _)
          new NegativePatternCall(body, Tuples.flatTupleOf(var_self, var___0_), MessageSendEvent.instance().getInternalQueryRepresentation());
          //     find messageReceiveEvent(self, _)
          new PositivePatternCall(body, Tuples.flatTupleOf(var_self, var___1_), MessageReceiveEvent.instance().getInternalQueryRepresentation());
          //     messageKind == MessageKind::found
          PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
          new ConstantValue(body, var__virtual_0_, org.eclipse.uml2.uml.MessageKind.get("found"));
          new Equality(body, var_messageKind, var__virtual_0_);
          bodies.add(body);
      }
      {
          PBody body = new PBody(this);
          PVariable var_self = body.getOrCreateVariableByName("self");
          PVariable var_messageKind = body.getOrCreateVariableByName("messageKind");
          PVariable var___0_ = body.getOrCreateVariableByName("_<0>");
          PVariable var___1_ = body.getOrCreateVariableByName("_<1>");
          new TypeConstraint(body, Tuples.flatTupleOf(var_self), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Message")));
          new TypeConstraint(body, Tuples.flatTupleOf(var_messageKind), new EDataTypeInSlotsKey((EDataType)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "MessageKind")));
          body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
             new ExportedParameter(body, var_self, parameter_self),
             new ExportedParameter(body, var_messageKind, parameter_messageKind)
          ));
          //     find messageSendEvent(self, _)
          new PositivePatternCall(body, Tuples.flatTupleOf(var_self, var___0_), MessageSendEvent.instance().getInternalQueryRepresentation());
          //     neg find messageReceiveEvent(self, _)
          new NegativePatternCall(body, Tuples.flatTupleOf(var_self, var___1_), MessageReceiveEvent.instance().getInternalQueryRepresentation());
          //     messageKind == MessageKind::lost
          PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
          new ConstantValue(body, var__virtual_0_, org.eclipse.uml2.uml.MessageKind.get("lost"));
          new Equality(body, var_messageKind, var__virtual_0_);
          bodies.add(body);
      }
      {
          PBody body = new PBody(this);
          PVariable var_self = body.getOrCreateVariableByName("self");
          PVariable var_messageKind = body.getOrCreateVariableByName("messageKind");
          PVariable var___0_ = body.getOrCreateVariableByName("_<0>");
          PVariable var___1_ = body.getOrCreateVariableByName("_<1>");
          new TypeConstraint(body, Tuples.flatTupleOf(var_self), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Message")));
          new TypeConstraint(body, Tuples.flatTupleOf(var_messageKind), new EDataTypeInSlotsKey((EDataType)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "MessageKind")));
          body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
             new ExportedParameter(body, var_self, parameter_self),
             new ExportedParameter(body, var_messageKind, parameter_messageKind)
          ));
          //     find messageSendEvent(self, _)
          new PositivePatternCall(body, Tuples.flatTupleOf(var_self, var___0_), MessageSendEvent.instance().getInternalQueryRepresentation());
          //     find messageReceiveEvent(self, _)
          new PositivePatternCall(body, Tuples.flatTupleOf(var_self, var___1_), MessageReceiveEvent.instance().getInternalQueryRepresentation());
          //     messageKind == MessageKind::complete
          PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
          new ConstantValue(body, var__virtual_0_, org.eclipse.uml2.uml.MessageKind.get("complete"));
          new Equality(body, var_messageKind, var__virtual_0_);
          bodies.add(body);
      }
      {
          PAnnotation annotation = new PAnnotation("Surrogate");
          annotation.addAttribute("feature", "messageKind");
          addAnnotation(annotation);
      }
      return bodies;
    }
  }
}
